=== modified file 'lib/lp/soyuz/model/archive.py'
--- lib/lp/soyuz/model/archive.py	2011-07-19 11:52:59 +0000
+++ lib/lp/soyuz/model/archive.py	2011-07-21 15:00:54 +0000
@@ -1532,11 +1532,21 @@
             [source], to_pocket, to_series, include_binaries,
             person=person)
 
+    def _checkCopyPackageFeatureFlags(self):
+        """Prevent copyPackage(s) if these conditions are not met."""
+        if not getFeatureFlag(u"soyuz.copypackage.enabled"):
+            raise ForbiddenByFeatureFlag
+        if (self.is_ppa and 
+            not getFeatureFlag(u"soyuz.copypackageppa.enabled")):
+            # We have no way of giving feedback about failed jobs yet,
+            # so this is disabled for now.
+            raise ForbiddenByFeatureFlag(
+                "Not enabled for copying to PPAs yet.")
+
     def copyPackage(self, source_name, version, from_archive, to_pocket,
                     person, to_series=None, include_binaries=False):
         """See `IArchive`."""
-        if not getFeatureFlag(u"soyuz.copypackage.enabled"):
-            raise ForbiddenByFeatureFlag
+        self._checkCopyPackageFeatureFlags()
 
         # Asynchronously copy a package using the job system.
         pocket = self._text_to_pocket(to_pocket)
@@ -1559,8 +1569,7 @@
     def copyPackages(self, source_names, from_archive, to_pocket,
                      person, to_series=None, include_binaries=None):
         """See `IArchive`."""
-        if not getFeatureFlag(u"soyuz.copypackage.enabled"):
-            raise ForbiddenByFeatureFlag
+        self._checkCopyPackageFeatureFlags()
 
         sources = self._collectLatestPublishedSources(
             from_archive, source_names)

=== modified file 'lib/lp/soyuz/tests/test_archive.py'
--- lib/lp/soyuz/tests/test_archive.py	2011-07-19 11:51:00 +0000
+++ lib/lp/soyuz/tests/test_archive.py	2011-07-21 15:00:54 +0000
@@ -1947,7 +1947,7 @@
     def test_copyPackage_requires_feature_flag(self):
         # Ensure feature is off.
         self.useFixture(FeatureFixture({u"soyuz.copypackage.enabled": ''}))
-        archive = self.factory.makeArchive()
+        archive = self.factory.makeArchive(purpose=ArchivePurpose.PRIMARY)
         self.assertRaises(
             ForbiddenByFeatureFlag,
             archive.copyPackage,
@@ -1956,7 +1956,27 @@
     def test_copyPackages_requires_feature_flag(self):
         # Ensure feature is off.
         self.useFixture(FeatureFixture({u"soyuz.copypackage.enabled": ''}))
-        archive = self.factory.makeArchive()
+        archive = self.factory.makeArchive(purpose=ArchivePurpose.PRIMARY)
+        self.assertRaises(
+            ForbiddenByFeatureFlag,
+            archive.copyPackages,
+            None, None, None, None, None)
+
+    def test_copyPackage_to_ppa_requires_feature_flag(self):
+        # Ensure feature is off.
+        self.useFixture(FeatureFixture({u"soyuz.copypackage.enabled": 'on'}))
+        self.useFixture(FeatureFixture({u"soyuz.copypackageppa.enabled": ''}))
+        archive = self.factory.makeArchive(purpose=ArchivePurpose.PPA)
+        self.assertRaises(
+            ForbiddenByFeatureFlag,
+            archive.copyPackage,
+            None, None, None, None, None)
+
+    def test_copyPackages_to_ppa_requires_feature_flag(self):
+        # Ensure feature is off.
+        self.useFixture(FeatureFixture({u"soyuz.copypackage.enabled": 'on'}))
+        self.useFixture(FeatureFixture({u"soyuz.copypackageppa.enabled": ''}))
+        archive = self.factory.makeArchive(purpose=ArchivePurpose.PPA)
         self.assertRaises(
             ForbiddenByFeatureFlag,
             archive.copyPackages,
@@ -1970,6 +1990,8 @@
     def setUp(self):
         super(TestSyncSource, self).setUp()
         self.useFixture(FeatureFixture({u"soyuz.copypackage.enabled": 'on'}))
+        self.useFixture(
+            FeatureFixture({u"soyuz.copypackageppa.enabled": 'on'}))
 
     def test_security_team_can_copy_to_primary(self):
         # A member of ubuntu-security can use syncSource on any package

